/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 16 mai 2007
   derniere modification : 05/07/2008 */

#ifndef SCEMODEL_H
#define SCEMODEL_H

#include <SCE/interface/SCENode.h>
#include <SCE/interface/SCEBoundingBox.h>
#include <SCE/interface/SCEMaterial.h>
#include <SCE/interface/SCEMesh.h>

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct sce_smodel SCE_SModel;

typedef unsigned int (*SCE_FMeshLODFunc)(SCE_SModel*, void*);
typedef void (*SCE_FModelRenderFunc)(SCE_SModel*, void*);

struct sce_smodel
{
    SCE_SMesh **meshs;    /* maillage du modele (tableau de lod) */
    unsigned int n_meshs; /* nombre de meshs */
    unsigned int lod;     /* LOD courant */
    int use_lod;          /* true si on utilise du LOD */
    SCE_FMeshLODFunc lodfunc; /* fonction de calcul du LOD */
    void *lodfuncparam;

    SCE_SMaterial *mat;   /* materiau du modele */

    int cullface;         /* true si backface culling active pour ce modele */

    SCE_SNode *node;      /* noeud du modele */
    SCE_SBoundingBox box; /* bounding box du modele */
    int have_box;
    /*SCE_SBoundingSphere sphere; */

    SCE_FModelRenderFunc render; /* fonction permettant le rendu du modele */
    void *render_param;          /* parametre de la fonction de rendu */

    int actived;          /* le modele est-il active pour le rendu ? */

    void *userdata;       /* donnees personnelles de l'utilisateur */
};


/* initialise le gestionnaire de modeles */
int SCE_Init_Model (void);
/* quitte le gestionnaire de modeles */
void SCE_Quit_Model (void);

/* initialise une structure de modele */
void SCE_Model_Init (SCE_SModel*);

/* cree un modele */
SCE_SModel* SCE_Model_Create (void);
/* supprime un modele */
void SCE_Model_Delete (SCE_SModel*);

/* fonctions d'assignation/recuperation des donnees personnelles */
void SCE_Model_SetData (SCE_SModel*, void*);
void* SCE_Model_GetData (SCE_SModel*);

/* defini les meshs d'un modele */
void SCE_Model_SetMeshs (SCE_SModel*, SCE_SMesh**, int);

/* charge un mesh */
int SCE_Model_LoadMesh (SCE_SModel*, const char*);

/* defini la fonction de rendu */
void SCE_Model_SetRenderFunc (SCE_SModel*, SCE_FModelRenderFunc, void*);
/* met en place la fonction de rendu par default d'un modele */
void SCE_Model_SetDefaultRenderFunc (SCE_SModel*);
/* indique si un modele possede une fonction de rendu */
int SCE_Model_HaveRenderFunc (SCE_SModel*);

/* active/desactive le backface culling */
void SCE_Model_ActivateBackfaceCulling (SCE_SModel*, int);
void SCE_Model_EnableBackfaceCulling (SCE_SModel*);
void SCE_Model_DisableBackfaceCulling (SCE_SModel*);

/* defini si on utilise du LOD pour un modele */
void SCE_Model_ActivateMeshLOD (SCE_SModel*, int);
void SCE_Model_EnableMeshLOD (SCE_SModel*);
void SCE_Model_DisableMeshLOD (SCE_SModel*);
/* determine la fonction de calcul du LOD pour un modele */
void SCE_Model_SetMeshLODFunc (SCE_SModel*, SCE_FMeshLODFunc, void*);
/* met a jour le niveau de detail */
void SCE_Model_UpdateMeshLOD (SCE_SModel*);

/* active/desactive un modele */
void SCE_Model_Activate (SCE_SModel*, int);
void SCE_Model_Enable (SCE_SModel*);
void SCE_Model_Disable (SCE_SModel*);
/* indique si un modele est active */
int SCE_Model_IsActived (SCE_SModel*);

/* retourne le niveau de LOD actif */
unsigned int SCE_Model_GetMeshLOD (SCE_SModel*);
/* retourne le noeud d'un modele */
SCE_SNode* SCE_Model_GetNode (SCE_SModel*);
/* retourne la bounding box d'un modele */
SCE_SBoundingBox* SCE_Model_GetBoundingBox (SCE_SModel*);
/* retourne le ou les meshs d'un modele */
SCE_SMesh** SCE_Model_GetMeshs (SCE_SModel*);
SCE_SMesh* SCE_Model_GetMesh (SCE_SModel*);
/* retourne le materiau d'un mesh */
SCE_SMaterial* SCE_Model_GetMaterial (SCE_SModel*);

/* indique si un modele possede une bounding box */
int SCE_Model_HaveBoundingBox (SCE_SModel*);
/* genere la bounding box d'un modele */
int SCE_Model_GenerateBoundingBox (SCE_SModel*);

/* bind un modele (sauf son noeud) */
void SCE_Model_Bind (SCE_SModel*);
/* bind le mesh d'un modele */
void SCE_Model_BindMesh (SCE_SModel*);
/* bind le materiau d'un modele */
void SCE_Model_BindMaterial (SCE_SModel*);
/* bind le noeud d'un modele */
void SCE_Model_BindNode (SCE_SModel*);
/* dessine le modele binde */
void SCE_Model_Draw (void);
/* fonction par defaut de rendu d'un modele */
void SCE_Model_RenderFunc (SCE_SModel*, void*);
/* effectue le rendu d'un modele */
void SCE_Model_Render (SCE_SModel*);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
