/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 26/02/2008
   derniere modification : 27/02/2008 */

#include <SCE/utils/SCERectangle.h>

/* ajoute le 26/02/2008 */
void SCE_Rectangle_Init (SCE_SIntRect *r)
{
    r->x1 = r->y1 = r->x2 = r->y2 = 0;
}
void SCE_Rectangle_Initf (SCE_SFloatRect *r)
{
    r->x1 = r->y1 = r->x2 = r->y2 = 0.0f;
}

/* ajoute le 27/02/2008 */
void SCE_Rectangle_Set (SCE_SIntRect *r, int x1, int y1, int x2, int y2)
{
    r->x1 = x1; r->x2 = x2; r->y1 = y1; r->y2 = y2;
}
void SCE_Rectangle_Setf (SCE_SFloatRect *r, float x1, float y1,
                         float x2, float y2)
{
    r->x1 = x1; r->x2 = x2; r->y1 = y1; r->y2 = y2;
}

/* ajoute le 27/02/2008 */
void SCE_Rectangle_Move (SCE_SIntRect *r, int x, int y)
{
    r->x1 += x; r->x2 += x;
    r->y1 += y; r->y2 += y;
}
void SCE_Rectangle_Movef (SCE_SFloatRect *r, float x, float y)
{
    r->x1 += x; r->x2 += x;
    r->y1 += y; r->y2 += y;
}
void SCE_Rectangle_Movev (SCE_SIntRect *r, int *v)
{
    SCE_Rectangle_Move (r, v[0], v[1]);
}
void SCE_Rectangle_Movefv (SCE_SFloatRect *r, SCE_TVector2 v)
{
    SCE_Rectangle_Movef (r, v[0], v[1]);
}

/* ajoute le 26/02/2008 */
int SCE_Rectangle_GetArea (SCE_SIntRect *r)
{
    return (r->x2 - r->x1) * (r->y2 - r->y1);
}
float SCE_Rectangle_GetAreaf (SCE_SFloatRect *r)
{
    return (r->x2 - r->x1) * (r->y2 - r->y1);
}

/* ajoute le 27/02/2008 */
int SCE_Rectangle_GetWidth (SCE_SIntRect *r)
{
    return r->x2 - r->x1;
}
float SCE_Rectangle_GetWidthf (SCE_SFloatRect *r)
{
    return r->x2 - r->x1;
}
/* ajoute le 27/02/2008 */
int SCE_Rectangle_GetHeight (SCE_SIntRect *r)
{
    return r->y2 - r->y1;
}
float SCE_Rectangle_GetHeightf (SCE_SFloatRect *r)
{
    return r->y2 - r->y1;
}

/* ajoute le 27/02/2008 */
int SCE_Rectangle_Equal (SCE_SIntRect *a, SCE_SIntRect *b)
{
    return (a->x1==b->x1 && a->y1==b->y1 && a->x2==b->x2 && a->y2==b->y2);
}
int SCE_Rectangle_Equalf (SCE_SFloatRect *a, SCE_SFloatRect *b)
{
#define EPS 0.000002 /* pux */
    return (fabs (a->x1 - b->x1) < EPS && fabs (a->y1 - b->y1) < EPS &&
            fabs (a->x2 - b->x2) < EPS && fabs (a->y2 - b->y2) < EPS);
}

/* ajoute le 26/02/2008 */
int SCE_Rectangle_IsIn (SCE_SIntRect *r, int x, int y)
{
    return (x > r->x1 && x < r->x2 && y > r->y1 && x < r->y2);
}
int SCE_Rectangle_IsInf (SCE_SFloatRect *r, float x, float y)
{
    return (x > r->x1 && x < r->x2 && y > r->y1 && x < r->y2);
}
int SCE_Rectangle_IsInv (SCE_SIntRect *r, int *p)
{
    return SCE_Rectangle_IsIn (r, p[0], p[1]);
}
int SCE_Rectangle_IsInfv (SCE_SFloatRect *r, float *p)
{
    return SCE_Rectangle_IsInf (r, p[0], p[1]);
}

/* ajoute le 27/02/2008 */
void SCE_Rectangle_MakeLines (SCE_SIntRect *re, SCE_SLine *l, SCE_SLine *r,
                              SCE_SLine *b, SCE_SLine *t)
{
    SCE_Line_Set (l, (int)re->x1, (int)re->y1, (int)re->x1, (int)re->y2);
    SCE_Line_Set (r, (int)re->x2, (int)re->y1, (int)re->x2, (int)re->y2);
    SCE_Line_Set (b, (int)re->x1, (int)re->y1, (int)re->x2, (int)re->y1);
    SCE_Line_Set (t, (int)re->x1, (int)re->y2, (int)re->x2, (int)re->y2);
}
void SCE_Rectangle_MakeLinesf (SCE_SFloatRect *re, SCE_SLine *l, SCE_SLine *r,
                               SCE_SLine *b, SCE_SLine *t)
{
    SCE_Line_Set (l, re->x1, re->y1, re->x1, re->y2);
    SCE_Line_Set (r, re->x2, re->y1, re->x2, re->y2);
    SCE_Line_Set (b, re->x1, re->y1, re->x2, re->y1);
    SCE_Line_Set (t, re->x1, re->y2, re->x2, re->y2);
}
void SCE_Rectangle_MakeLinesv (SCE_SIntRect *r, SCE_SLine *lines)
{
    SCE_Rectangle_MakeLines (r, &lines[0], &lines[1], &lines[2], &lines[3]);
}
void SCE_Rectangle_MakeLinesfv (SCE_SFloatRect *r, SCE_SLine *lines)
{
    SCE_Rectangle_MakeLinesf (r, &lines[0], &lines[1], &lines[2], &lines[3]);
}

/* ajoute le 26/02/2008 */
int SCE_Rectangle_Intersects (SCE_SIntRect *a, SCE_SIntRect *b)
{
    int start[] = {max (a->x1, b->x1), max (a->y1, b->y1)};
    int end[]   = {min (a->x2, b->x2), min (a->y2, b->y2)};
    SCE_SIntRect r = {start[0], start[1], end[0], end[1]};

    if ((start[0] > end[0]) || (start[1] > end[1]))
        return SCE_RECT_OUT;
    else if (SCE_Rectangle_Equal (&r, a) || SCE_Rectangle_Equal (&r, b))
        return SCE_RECT_IN;
    else
        return SCE_RECT_INTERSECTS;
}
int SCE_Rectangle_Intersectsf (SCE_SFloatRect *a, SCE_SFloatRect *b)
{
    SCE_TVector2 start = {max (a->x1, b->x1), max (a->y1, b->y1)};
    SCE_TVector2 end   = {min (a->x2, b->x2), min (a->y2, b->y2)};
    SCE_SFloatRect r = {start[0], start[1], end[0], end[1]};

    if ((start[0] > end[0]) || (start[1] > end[1]))
        return SCE_RECT_OUT;
    else if (SCE_Rectangle_Equalf (&r, a) || SCE_Rectangle_Equalf (&r, b))
        return SCE_RECT_IN;
    else
        return SCE_RECT_INTERSECTS;
}
